@@ -57,18 +57,20 @@ public:
     Constructor.
 
     \param N Number of particles.
-    \param y Initial yield positions.
+    \param y Initial yield positions [#N, n_yield].
     */
-    System(size_t N, const xt::xtensor<double, 2>& y);
+    template <class T>
+    System(size_t N, const T& y);
 
     /**
     Constructor.
 
     \param N Number of particles.
-    \param y Initial yield positions.
-    \param istart Starting index corresponding to y[:, 0]
+    \param y Initial yield positions [#N, n_yield].
+    \param istart Starting index corresponding to y[:, 0], [#N].
     */
-    System(size_t N, const xt::xtensor<double, 2>& y, const xt::xtensor<long, 1>& istart);
+    template <class T, class I>
+    System(size_t N, const T& y, const I& istart);
 
     /**
     Number of particles.
@@ -88,8 +90,8 @@ public:
     /**
     \copydoc QPot::Chunked::set_y(long, const T&)
     */
-    template <class T>
-    void set_y(const xt::xtensor<long, 1>& istart, const T& y);
+    template <class I, class T>
+    void set_y(const I& istart, const T& y);
 
     /**
     \copydoc QPot::Chunked::set_y(long, const T&)
@@ -171,7 +173,8 @@ public:
     \param x Trial particle positions (internally the position is not updated).
     \return true if redraw is needed for one of more particle.
     */
-    bool any_redraw(const xt::xtensor<double, 1>& x) const;
+    template <class T>
+    bool any_redraw(const T& x) const;
 
     /**
     Check if any particle if within `n` potentials for the left- or the right-most yield
@@ -268,7 +271,8 @@ public:
 
     \param arg The particles' positions [#N].
     */
-    void set_x(const xt::xtensor<double, 1>& arg);
+    template <class T>
+    void set_x(const T& arg);
 
     /**
     Set the velocity of each particle.
@@ -277,14 +281,16 @@ public:
 
     \param arg The particles' velocities [#N].
     */
-    void set_v(const xt::xtensor<double, 1>& arg);
+    template <class T>
+    void set_v(const T& arg);
 
     /**
     Set the acceleration of each particle.
 
     \param arg The particles' accelerations [#N].
     */
-    void set_a(const xt::xtensor<double, 1>& arg);
+    template <class T>
+    void set_a(const T& arg);
 
     /**
     Position of each particle.
@@ -342,6 +348,13 @@ public:
     */
     xt::xtensor<double, 1> f_damping() const;
 
+    /**
+    The time step, see set_dt().
+
+    \return double.
+    */
+    double dt() const;
+
     /**
     Residual: the ratio between the norm of #f and #f_frame.
 
@@ -378,18 +391,90 @@ public:
     size_t minimise(double tol = 1e-5, size_t niter_tol = 10, size_t max_iter = 1000000);
 
     /**
-    Event driven advance right to closest yielding point, leaving ``delta_x / 2`` as margin.
+    Minimise energy: run timeStep() until a mechanical equilibrium has been reached.
+    Compared to minimise() this function measures the duration of an avalanche.
 
-    \param delta_x Margin.
+    \param tol
+        Relative force tolerance for equilibrium. See residual() for definition.
+
+    \param niter_tol
+        Enforce the residual check for ``niter_tol`` consecutive increments.
+
+    \param max_iter
+        Maximum number of iterations. Throws ``std::runtime_error`` otherwise.
+
+    \return The number of iterations elapsed during an avalanche (in units of dt()).
     */
-    void advanceRightElastic(double delta_x);
+    size_t minimise_timeactivity(double tol = 1e-5, size_t niter_tol = 10, size_t max_iter = 1000000);
 
     /**
-    Event driven: advance right by ``delta_x``.
+    Advance the system elastically: the particles and the frame are moved proportionally,
+    such that equilibrium is maintained.
+    Note that the displacement is uniform: all particles are moved in the same way.
+    In particular:
 
-    \param delta_x Step size.
+    \f$ \Delta f^\mathrm{potential}_i = \mu \Delta x_i \f$
+
+    \f$ \Delta f^\mathrm{frame} = k (\Delta x^\mathrm{frame} - \Delta x_i) \f$
+
+    This gives two cases:
+
+    -   Given \f$ \Delta x_i \f$ one finds:
+        \f$ \Delta x^\mathrm{frame} = (k + \mu) / k \Delta x_i \f$.
+
+    -   Given \f$ \Delta x^\mathrm{frame} \f$ one finds:
+        \f$ \Delta x_i = k / (k + \mu) \Delta x^\mathrm{frame} \f$.
+
+    \param dx Displacement.
+
+    \param dx_of_frame
+        If `true`: `dx` \f$ = \Delta x^\mathrm{frame} \f$,
+        if `false`: `dx` \f$ = \Delta x_i \f$,
     */
-    void advanceRightKick(double delta_x);
+    void advanceElastic(double dx, bool dx_of_frame = true);
+
+    /**
+    Event driven advance right to closest yielding point, leaving ``eps / 2`` as margin.
+
+    \param eps Margin.
+    */
+    void advanceEventRightElastic(double eps);
+
+    /**
+    Event driven: advance right by ``eps``.
+
+    \param eps Step size.
+    */
+    void advanceEventRightKick(double eps);
+
+    /**
+    \cond
+    */
+    [[ deprecated ]]
+    void advanceRightElastic(double eps);
+
+    [[ deprecated ]]
+    void advanceRightKick(double eps);
+    /**
+    \endcond
+    */
+
+    /**
+    Trigger a specific particle:
+    advance to the yield positions right plus a margin of `eps / 2`.
+
+    \param p Particle index.
+    \param eps Margin.
+    */
+    void triggerRight(size_t p, double eps);
+
+    /**
+    Trigger the closest point to yielding right:
+    advance to the yield positions right plus a margin of `eps / 2`.
+
+    \param eps Margin.
+    */
+    void triggerWeakestRight(double eps);
 
 protected:
 
@@ -400,7 +485,8 @@ protected:
     \param y Initial yield positions.
     \param istart Starting index corresponding to y[:, 0]
     */
-    void init(size_t N, const xt::xtensor<double, 2>& y, const xt::xtensor<long, 1>& istart);
+    template <class T, class I>
+    void init(size_t N, const T& y, const I& istart);
 
     /**
     Compute #f based on the current #x and #v.
