@@ -30,26 +30,25 @@ inline std::vector<std::string> version_dependencies()
     return ret;
 }
 
-inline System::System(size_t N, const xt::xtensor<double, 2>& x_y)
+template <class T>
+inline System::System(size_t N, const T& x_y)
 {
     xt::xtensor<long, 1> istart = xt::zeros<long>({N});
     this->init(N, x_y, istart);
 }
 
-inline System::System(
-    size_t N,
-    const xt::xtensor<double, 2>& x_y,
-    const xt::xtensor<long, 1>& istart)
+template <class T, class I>
+inline System::System(size_t N, const T& x_y, const I& istart)
 {
     this->init(N, x_y, istart);
 }
 
-inline void System::init(
-    size_t N,
-    const xt::xtensor<double, 2>& x_y,
-    const xt::xtensor<long, 1>& istart)
+template <class T, class I>
+inline void System::init(size_t N, const T& x_y, const I& istart)
 {
     #ifdef FRICTIONQPOTSPRINGBLOCK_ENABLE_ASSERT
+    FRICTIONQPOTSPRINGBLOCK_ASSERT(x_y.dimension() == 2);
+    FRICTIONQPOTSPRINGBLOCK_ASSERT(istart.dimension() == 1);
     auto y0 = xt::view(x_y, xt::all(), 0);
     FRICTIONQPOTSPRINGBLOCK_ASSERT(xt::all(y0 < 0.0));
     #endif
@@ -84,10 +83,10 @@ inline QPot::Chunked& System::y(size_t p)
     return m_y[p];
 }
 
-template <class T>
-inline void System::set_y(const xt::xtensor<long, 1>& istart, const T& x_y)
+template <class I, class T>
+inline void System::set_y(const I& istart, const T& x_y)
 {
-    FRICTIONQPOTSPRINGBLOCK_ASSERT(istart.size() == m_N);
+    FRICTIONQPOTSPRINGBLOCK_ASSERT(xt::has_shape(istart, {m_N}));
     FRICTIONQPOTSPRINGBLOCK_ASSERT(x_y.dimension() == 2);
     FRICTIONQPOTSPRINGBLOCK_ASSERT(x_y.shape(0) == m_N);
 
@@ -231,9 +230,10 @@ inline bool System::any_redraw() const
     return false;
 }
 
-inline bool System::any_redraw(const xt::xtensor<double, 1>& xtrial) const
+template <class T>
+inline bool System::any_redraw(const T& xtrial) const
 {
-    FRICTIONQPOTSPRINGBLOCK_ASSERT(xtrial.size() == m_N);
+    FRICTIONQPOTSPRINGBLOCK_ASSERT(xt::has_shape(xtrial, {m_N}));
 
     for (size_t p = 0; p < m_N; ++p) {
         auto r = m_y[p].redraw(xtrial(p));
@@ -319,9 +319,10 @@ inline void System::set_x_frame(double arg)
     this->computeForce();
 }
 
-inline void System::set_x(const xt::xtensor<double, 1>& arg)
+template <class T>
+inline void System::set_x(const T& arg)
 {
-    FRICTIONQPOTSPRINGBLOCK_ASSERT(arg.size() == m_N);
+    FRICTIONQPOTSPRINGBLOCK_ASSERT(xt::has_shape(arg, {m_N}));
     xt::noalias(m_x) = arg;
     this->computeForcePotential();
     this->computeForceNeighbours();
@@ -329,17 +330,19 @@ inline void System::set_x(const xt::xtensor<double, 1>& arg)
     this->computeForce();
 }
 
-inline void System::set_v(const xt::xtensor<double, 1>& arg)
+template <class T>
+inline void System::set_v(const T& arg)
 {
-    FRICTIONQPOTSPRINGBLOCK_ASSERT(arg.size() == m_N);
+    FRICTIONQPOTSPRINGBLOCK_ASSERT(xt::has_shape(arg, {m_N}));
     xt::noalias(m_v) = arg;
     this->computeForceDamping();
     this->computeForce();
 }
 
-inline void System::set_a(const xt::xtensor<double, 1>& arg)
+template <class T>
+inline void System::set_a(const T& arg)
 {
-    FRICTIONQPOTSPRINGBLOCK_ASSERT(arg.size() == m_N);
+    FRICTIONQPOTSPRINGBLOCK_ASSERT(xt::has_shape(arg, {m_N}));
     xt::noalias(m_a) = arg;
 }
 
@@ -416,6 +419,11 @@ inline void System::timeStep()
     }
 }
 
+inline double System::dt() const
+{
+    return m_dt;
+}
+
 inline double System::residual() const
 {
     double Fext = xt::norm_l2(m_f_frame)();
@@ -427,6 +435,8 @@ inline void System::quench()
 {
     m_v.fill(0.0);
     m_a.fill(0.0);
+    this->computeForceDamping();
+    this->computeForce();
 }
 
 inline size_t System::minimise(double tol, size_t niter_tol, size_t max_iter)
@@ -446,32 +456,126 @@ inline size_t System::minimise(double tol, size_t niter_tol, size_t max_iter)
     throw std::runtime_error("No convergence found");
 }
 
+inline size_t System::minimise_timeactivity(double tol, size_t niter_tol, size_t max_iter)
+{
+    GooseFEM::Iterate::StopList stop(niter_tol);
+
+    auto i_n = this->i();
+    long s = 0;
+    long s_n = 0;
+    size_t first_iter = 0;
+    size_t last_iter = 0;
+    bool init = true;
+
+    for (size_t iiter = 0; iiter < max_iter; ++iiter) {
+
+        this->timeStep();
+
+        s = 0;
+
+        for (size_t p = 0; p < m_N; ++p) {
+            s += std::abs(m_y[p].i() - i_n(p));
+        }
+
+        if (s != s_n) {
+            if (init) {
+                init = false;
+                first_iter = iiter;
+            }
+            last_iter = iiter;
+        }
+
+        s_n = s;
+
+        if (stop.stop(this->residual(), tol)) {
+            this->quench();
+            return last_iter - first_iter;
+        }
+    }
+
+    throw std::runtime_error("No convergence found");
+}
+
+/**
+\cond
+*/
+
 inline void System::advanceRightElastic(double eps)
+{
+    FRICTIONQPOTSPRINGBLOCK_WARNING_PYTHON("rename 'advanceRightElastic' -> 'advanceEventRightElastic'");
+    this->advanceEventRightElastic(eps);
+}
+
+inline void System::advanceRightKick(double eps)
+{
+    FRICTIONQPOTSPRINGBLOCK_WARNING_PYTHON("rename 'advanceRightKick' -> 'advanceEventRightKick'");
+    this->advanceEventRightKick(eps);
+}
+
+/**
+\endcond
+*/
+
+inline void System::advanceEventRightElastic(double eps)
 {
     double dx = xt::amin(this->yieldDistanceRight())();
     if (dx < eps / 2.0) {
         return;
     }
-    double deltax = dx - eps / 2.0;
-    m_x += deltax;
-    m_x_frame += (deltax * m_mu / m_k_frame);
+    this->advanceElastic(dx - eps / 2.0, false);
+}
+
+inline void System::advanceEventRightKick(double eps)
+{
+    this->advanceElastic(eps, false);
+}
+
+inline void System::advanceElastic(double dx, bool dx_of_frame)
+{
+    double dx_particles;
+    double dx_frame;
+
+    if (dx_of_frame) {
+        dx_frame = dx;
+        dx_particles = dx * m_k_frame / (m_k_frame + m_mu);
+    }
+    else {
+        dx_particles = dx;
+        dx_frame = dx * (m_k_frame + m_mu) / m_k_frame;
+    }
+
+    #ifdef FRICTIONQPOTSPRINGBLOCK_ENABLE_DEBUG
+        if (dx_particles > 0.0) {
+            FRICTIONQPOTSPRINGBLOCK_DEBUG(dx_particles < xt::amin(this->yieldDistanceRight())());
+        }
+        else {
+            FRICTIONQPOTSPRINGBLOCK_DEBUG(std::abs(dx_particles) < xt::amin(this->yieldDistanceLeft())());
+        }
+    #endif
+
+    m_x += dx_particles;
+    m_x_frame += dx_frame;
     this->computeForcePotential();
     this->computeForceNeighbours();
     this->computeForceFrame();
     this->computeForce();
 }
 
-inline void System::advanceRightKick(double eps)
+inline void System::triggerRight(size_t p, double eps)
 {
-    double deltax = eps;
-    m_x += deltax;
-    m_x_frame += (deltax * m_mu / m_k_frame);
+    FRICTIONQPOTSPRINGBLOCK_ASSERT(p < m_N);
+    m_x(p) = m_y[p].yright() + eps / 2.0;
     this->computeForcePotential();
     this->computeForceNeighbours();
     this->computeForceFrame();
     this->computeForce();
 }
 
+inline void System::triggerWeakestRight(double eps)
+{
+    this->triggerRight(xt::argmin(this->yieldDistanceRight())(), eps);
+}
+
 inline double System::x_frame() const
 {
     return m_x_frame;
